#!/bin/sh
# enrolls a new certificate, possibly using the existing certificate

MOD_NAME=scepClient
SCEP_BIN=/opt/$MOD_NAME/bin/sscep
SCEP_DATA=/var/data/$MOD_NAME
SCEP_LOG=/tmp/$MOD_NAME.log
OPENSSL_CONF=/tmp/$MOD_NAME.cnf
SYSLOG="/usr/bin/logger -t $MOD_NAME"

. /opt/$MOD_NAME/etc/settings

# template subsitution
SN=`status sys | awk '/Serial Number/ {print $4}'`
CERT_SUBJECT=${MOD_SCEPCLIENT_CERT_SUBJECT//\{SN\}/$SN}

$SYSLOG "enrollment started"
mkdir -p $SCEP_DATA

echo "oid_section        = new_oids"                   > $OPENSSL_CONF

echo "[ req ]"                                         >> $OPENSSL_CONF
echo "default_bits       = $MOD_SCEPCLIENT_CERT_BITS"  >> $OPENSSL_CONF
echo "default_keyfile    = $SCEP_DATA/new.key"         >> $OPENSSL_CONF
echo "encrypt_key        = no"                         >> $OPENSSL_CONF
echo "distinguished_name = req_dn"                     >> $OPENSSL_CONF
echo "attributes         = req_attributes"             >> $OPENSSL_CONF
echo "req_extensions     = req_ext"                    >> $OPENSSL_CONF

echo "[ new_oids ]"                                    >> $OPENSSL_CONF
# this is a Microsoft specific extension used also by the OpenXPKI
echo "certTemplateName   = 1.3.6.1.4.1.311.20.2"  >> $OPENSSL_CONF

echo "[ req_dn ]"                                      >> $OPENSSL_CONF
echo "[ req_attributes ]"                              >> $OPENSSL_CONF

echo "[ req_ext ]"                                     >> $OPENSSL_CONF
echo "basicConstraints   = critical, CA:FALSE"         >> $OPENSSL_CONF

if [ "$MOD_SCEPCLIENT_USAGE_DIGITAL_SIGNATURE" == "1" ]; then KEY_USAGE="$KEY_USAGE, digitalSignature"; fi
if [ "$MOD_SCEPCLIENT_USAGE_KEY_ENCIPHERMENT"  == "1" ]; then KEY_USAGE="$KEY_USAGE, keyEncipherment"; fi
if [ "$MOD_SCEPCLIENT_USAGE_SERVER_AUTH"       == "1" ]; then EXKEY_USAGE="$EXKEY_USAGE, serverAuth"; fi
if [ "$MOD_SCEPCLIENT_USAGE_CLIENT_AUTH"       == "1" ]; then EXKEY_USAGE="$EXKEY_USAGE, clientAuth"; fi
echo "keyUsage           = critical, ${KEY_USAGE:2}"   >> $OPENSSL_CONF
echo "extendedKeyUsage   = critical, ${EXKEY_USAGE:2}" >> $OPENSSL_CONF

if [ ! -z "$MOD_SCEPCLIENT_CERT_ALT_NAME" ]; then
  echo "subjectAltName   = $MOD_SCEPCLIENT_CERT_ALT_NAME" >> $OPENSSL_CONF
fi

if [ ! -z "$MOD_SCEPCLIENT_CERT_TEMPLATE" ]; then
  echo "certTemplateName = ASN1:UTF8String:$MOD_SCEPCLIENT_CERT_TEMPLATE" >> $OPENSSL_CONF
fi

openssl req -new -config $OPENSSL_CONF -subj "$CERT_SUBJECT" -out /tmp/$MOD_NAME.csr > $SCEP_LOG 2>&1
retval="$?"
if [ $retval -ne 0 ]; then
  $SYSLOG "failed to build a certificate signing request ($retval)"
  [ -f /var/scripts/scep-failure ] && . /var/scripts/scep-failure
  exit 1
fi

rm -f $OPENSSL_CONF

$SCEP_BIN getca -c $SCEP_DATA/ca.crt -u $MOD_SCEPCLIENT_SERVER_URL -W 10 -v >> $SCEP_LOG 2>&1
retval="$?"
if [ $retval -ne 0 ]; then
  tail -1 $SCEP_LOG | $SYSLOG
  $SYSLOG "failed to obtain CA certificates ($retval)"
  [ -f /var/scripts/scep-failure ] && . /var/scripts/scep-failure
  exit 1
fi

if [ -f $SCEP_DATA/latest.crt ]; then
  oldserial=`ls -l $SCEP_DATA/latest.crt | awk '{print $11}'`
  $SYSLOG "supersede certificate ${oldserial:0:-4}," `openssl x509 -in $SCEP_DATA/latest.crt -enddate -noout`
  # will sign the request using our certificates, so manual approval is not needed
  CURRENT_CERTS="-K $SCEP_DATA/latest.key -O $SCEP_DATA/latest.crt"
fi
$SCEP_BIN enroll -u $MOD_SCEPCLIENT_SERVER_URL $CURRENT_CERTS \
  -k $SCEP_DATA/new.key -r /tmp/$MOD_NAME.csr -l $SCEP_DATA/new.crt -c $SCEP_DATA/ca.crt \
  -t $MOD_SCEPCLIENT_AWAIT -T $((60*MOD_SCEPCLIENT_AWAIT_MAX)) -n 2000000000 -W 10 -v >> $SCEP_LOG 2>&1
retval="$?"
if [ $retval -ne 0 ]; then
  tail -1 $SCEP_LOG | $SYSLOG
  $SYSLOG "failed to enroll a certificate ($retval)"
  [ -f /var/scripts/scep-failure ] && . /var/scripts/scep-failure
  exit 1
fi

# evaluate serial=xxxxxx to retrieve the certificate serial number
eval `openssl x509 -in $SCEP_DATA/new.crt -noout -serial`

rm -f $SCEP_LOG
$SYSLOG "enrollment successful $serial"

# keep track of the latest key/certificate, but never delete the obsolete
mv $SCEP_DATA/new.key $SCEP_DATA/$serial.key
mv $SCEP_DATA/new.crt $SCEP_DATA/$serial.crt
ln -sf $serial.key $SCEP_DATA/latest.key
ln -sf $serial.crt $SCEP_DATA/latest.crt

[ -f /var/scripts/scep-success ] && . /var/scripts/scep-success
